#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
EXTPROGNAME="smhastatus"

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh

PARM_FLAGS="h help d detail r refresh m mirrors"
PARM_MULTI=""
PARM_TRANSLATION="r refresh \
d detail \
m mirrors \
h help"
PARM_SINGLES="help"
PARM_VALID=$PARM_TRANSLATION

LOOP_INTERVAL=20

###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    #list_parm_vars 
    
    if [ -n "$mirrors" ]; then
        ensure_parms_not "mirrors" "detail"
    fi
     
    
    
}

echo_help () {
	echo -e "`eval_gettext \"\\\${EXTPROGNAME}: [-h|--help][-r|--refresh][-d|--detail][-m|--mirrors]\"`"
    echo -e "`eval_gettext \"The \\\${EXTPROGNAME} command displays the status of the high availability environment.\"`"
    echo -e "`eval_gettext "Options:\n"`"
    echo -e "`eval_gettext "\t-h,--help\tDisplay this help"`"
    echo -e "`eval_gettext "\t-d,--detail\tShow detailed status"`"
    echo -e "`eval_gettext "\t-m,--mirrors\tShow mirroring status"`"
    echo -e "`eval_gettext "\t-r,--refresh\tContinuously display the status if it changes\n"`"
}

#####################################################
# Process the help parameter
#####################################################
proc_help () {
    
    if [[ "$help" == "Y" ]]; then
        echo_help
        exit 0
    fi

}
#####################################################
# Process the lssam status
#####################################################
proc_lssam () {
    local lssam_refresh=
    if [ -n "$refresh" ]; then
        lssam_refresh="-top"
    fi
    
    lssam $lssam_refresh
}

is_config_in_progress_2 () {
    local RC=0
    STATUS_LINE=
    # First check if the daemon file is present
    if [ -f $ERROR_DURING_CONFIG_FLAGFILE ]; then
        # HA config has been attempted, and an error occurred
        RC=4
        if [ -f $CFG_STATUS_FILE ]; then
            status_key=`tail -n 1 $CFG_STATUS_FILE`
            STATUS_LINE="`eval_gettext "$status_key"`"
        else
            STATUS_LINE="`eval_gettext "Error"`"
        fi 
    else 
    
        # First check if the daemon file is present
        if [ -f $DM_CONF -o -f $BROKER_KS ]; then
            STATUS_LINE="`eval_gettext "Configuration is starting."`"
            RC=5
        fi

        if [ -f $CFG_STATUS_FILE ]; then
            status_key=`tail -n 1 $CFG_STATUS_FILE`
            STATUS_LINE="`eval_gettext "$status_key"`"
            RC=5
        fi
    fi
 
    return $RC              
}

proc_ha_status___new () {
    local RC=0
    local do_refresh=1
    local last_state="DONTKNOW"
    local state
    local newline=1
    local previous_newline
    local stat_RC
    local check_RC

    until [ $do_refresh -eq 0 ]; do

        # Save the previous newline state and reset it.
        previous_newline=$newline
        newline=1

        # First check if HA is configured
        is_HAconfigured
        if [ $? -eq 0 ]; then
            # This node is not fully configured, see if it's in progress
            is_config_in_progress_2
            local progress_rc=$?
            local last_status=$STATUS_LINE

            if [ $progress_rc -ne 0 ]; then
                # This node is being configured
                state="\r$last_status"
                if [ $progress_rc -eq 4 ]; then
                    # Set RC to 4, which means Not Configured, since an error occurred during configuration
                    RC=4
                else
                    # Set RC to 6, which means Configuration in Progress
                    RC=6
                fi
            else
                # This node is not configured           
                state="`eval_gettext "Not Configured"`"
                # Set RC to four, which means Not Configured
                RC=4
            fi
            
        else
            # So we think it's setup, but is it terminating?
            is_terminating
            if [ $? -eq 1 ]; then
                # The configuration is being removed
                state="`eval_gettext "Removing Configuration"`"
                # Set RC to 2 which means ending
                RC=2   
            else
                # Check if this node is passive            
                check_passive_state
                stat_RC=$?
                if [ $stat_RC -eq 0 -o $stat_RC -eq 1 ]; then
                 
                    # This is presumably the active or passive node. Need to check for sure

                    check_rgs  # 0=all RGs online, 1=all RGs offline, 2=RGs pending,
                               # 3=all RGs have their OpState matching their NominalState, 9=error
                    check_RC=$?

                    case $check_RC in
                        0|3) if [ $stat_RC -eq 0 ]
                           then 
                               state="`eval_gettext "Active"`"
                               # Set RC to zero, which means Active; this is the active node
                               RC=0    
                           else
                               state="`eval_gettext "Passive"`"
                               # Set RC to 3 which means inactive; this is truly the passive node
                               RC=3
                           fi ;;
                        1) state="`eval_gettext "Stopped"`"
                           # Set RC to 7 which means stopped
                           RC=7 ;;
                        2) state="`eval_gettext "Pending"`"
                           # Set RC to 8 which means pending (online or offline)
                           RC=8 ;;
                        *) if [ -e $SMHA_CONFIGDIR/.modelJustCreated ]
                           then
                               state="`eval_gettext "Pending"`"
                               # Set RC to 8 which means pending (online or offline)
                               RC=8
                           else
                               state="`eval_gettext "Error"`"
                               # Set RC to 1 which means error.
                               RC=1
                           fi ;;
                    esac
                elif [ $stat_RC -eq 2 ]; then
                    state="`eval_gettext "Pending"`"
                    debug "check_passive_state returned 2, so returning Pending (8) from smhastatus"
                    # Set RC to 8 which means pending (online or offline)
                    RC=8
                else
                    # there was a problem accessing RMC, check Peer Domain state
                    check_PD_status
                    stat_RC=$?

                    case $stat_RC in
                        1) state="`eval_gettext "Stopped"`"
                           # Set RC to 7 which means stopped
                           RC=7 ;;
                        2) state="`eval_gettext "Pending"`"
                           # Set RC to 8 which means pending (online or offline)
                           RC=8 ;;
                        *) state="`eval_gettext "Error"`"
                           # Set RC to 1 which means error.
                           # If the PD is online at this point (rc=0), then check_passive_state
                           # should not have failed. If the PD is not configured (rc=3), then
                           # is_HAconfigured should have returned false. Otherwise (rc=9), then
                           # there is a serious problem with RSCT
                           RC=1 ;;
                    esac
                fi
            fi
        fi
                
        # Check if we should output the state or not
        if [ "$last_state" != "$state" ]; then
            if [ $newline -eq 0 ]; then
                if [ -z "$refresh" ]; then
                    echo -e "$state   "
                else    
                    # Print the state w/o the \n
                    echo -en "$state   "
                fi
            else
                if [ $previous_newline -eq 0 ]; then
                    echo -e "\n$state"
                else
                    echo -e "$state"                
                fi
            fi
            
            # Record the last state
            last_state=$state
            
        fi
        
        
        # If the refresh flag is not set then don't loop again
        if [ -z "$refresh" ]; then
            do_refresh=0
        else
            # Sleep before looping
            sleep $LOOP_INTERVAL
        fi
    
    done
    
    exit $RC

}


#####################################################
# Process the ha mirror status
#####################################################
mirror_status () {
	local RC=0
    local do_refresh=1
        
    until [ $do_refresh -eq 0 ]; do

        if [ -n "$refresh" ]; then
        	clear
        fi

        mirror_header
        mirror_sync_status
 
        # If the refresh flag is not set then don't loop again
        if [ -z "$refresh" ]; then
            do_refresh=0
        else
            # Sleep before looping
            sleep $LOOP_INTERVAL
        fi
    
    done
    
    exit $RC
	
}

mirror_header () {
    echo "`eval_gettext "HA Configuration Mirror Status"`"
}

mirror_sync_status () {
    local tmpf=`mktemp /tmp/smha_mss.XXXXXX`  || fatal_error "Could not make temporary file"
    LC_ALL=C drbdadm status 2>/dev/null | grep -o -e "name=[A-Za-z0-9\"]*" -e "cs=[A-Za-z0-9\"]*" -e "ds1=[A-Za-z\"]*" -e "ds2=[A-Za-z\"]*" -e "percent=[0-9.\"]*" >> $tmpf
    
    name_read=
    line_outputted=
    
    while read -r line
    do
        case "$line" in
            name*)
                if [ -n "$name_read" ]; then
                    mirror_output_line
                fi
                name=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                name=/$name
                name_read=1
            ;;
            cs*)
                cs=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                translate_cs
            ;;
            ds1*)
                ds1=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                translate_ds $ds1
                ds1=$DSTATE
            ;;
            ds2*)
                ds2=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                translate_ds $ds2
                ds2=$DSTATE
            ;;
            percent*)
                percent=`echo $line | sed "s/\"//g" | cut -d'=' -f 2`
                mirror_output_line
            ;;

            *)
                debug "line not handled: $line"
            ;;
        esac
    done < $tmpf    
    rm $tmpf
    if [ -n "$name_read" ]; then
        mirror_output_line
    fi
    if [ -z "$line_outputted" ]; then
        echo "`eval_gettext "No status is available."`"
    fi
    
    
}
translate_cs () {
        case "$cs" in
            SyncTarget)
                cs="`eval_gettext "Synchronizing from other node"`"
            ;;
            SyncSource)
                cs="`eval_gettext "Synchronizing to other node"`"
            ;;
            Connected)
                cs="`eval_gettext "Connected"`"
            ;;
            Verif*)
                cs="`eval_gettext "Connected"`"
            ;;
            Paused*)
                cs="`eval_gettext "Paused"`"
            ;;
            WFConnection)
                cs="`eval_gettext "Waiting for connection"`"
            ;;
            StartingSyn*)
                cs="`eval_gettext "Starting synchronization"`"
            ;;
            Unconnected)
                cs="`eval_gettext "Not connected"`"
            ;;
            Timeout)
                cs="`eval_gettext "Not connected"`"
            ;;
            Disconnecting)
                cs="`eval_gettext "Not connected"`"
            ;;
            StandAlone)
                cs="`eval_gettext "Not connected"`"
            ;;
            Unconfigured)
                # DRBD shows Unconfigured sometimes when replication interface goes down, so map this to "Not connected"
                cs="`eval_gettext "Not connected"`"
            ;;
            *)
                debug "cs not recongized: $cs"
                cs="`eval_gettext "Error"`"
                
            ;;
        esac
	
}

translate_ds () {
    local dstate=$1
    	   
    case "$dstate" in
        Inconsistent)
            DSTATE="`eval_gettext "Inconsistent"`"
        ;;
        UpToDate)
            DSTATE="`eval_gettext "Up-to-date"`"
        ;;
        Outdated)
            DSTATE="`eval_gettext "Outdated"`"
        ;;
        Diskless)
            DSTATE="`eval_gettext "Error"`"
        ;;
        Attaching)
            DSTATE="`eval_gettext "Connecting"`"
        ;;
        Consistent)
            DSTATE="`eval_gettext "Consistent"`"
        ;;
        *)
            debug "dstate not recongized: $dstate"
            DSTATE="`eval_gettext "Unknown"`"
            
        ;;
    esac
    
}

mirror_output_line () {
    # if cs, ds1, ds2 are empty, fill them with translated equivalent of "Unknown"
    if [ "$cs" == "" ]; then
        cs="`eval_gettext "Unknown"`"
    fi
    if [ "$ds1" == "" ]; then
        ds1="`eval_gettext "Unknown"`"
    fi
    if [ "$ds2" == "" ]; then
        ds2="`eval_gettext "Unknown"`"
    fi
    
    echo "`eval_gettext \"Name: \\\$name\"`"
    echo "`eval_gettext \"    Connection status: \\\$cs\"`"
    echo "`eval_gettext \"    Status on this node: \\\$ds1\"`"
    echo "`eval_gettext \"    Status on other node: \\\$ds2\"`"
    if [ -n "$percent" ]; then
        echo "`eval_gettext \"    Synchronization percent: \\\$percent\"`"
    fi
    name=
    cs=
    ds1=
    ds2=
    percent=
    name_read=
    line_outputted=1

}


###################################
## Main code                     ##
###################################

trap 'exit 0' SIGINT SIGTERM

proc_parms $*
##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help

if [ -n "$mirrors" ]; then
    mirror_status
fi

if [ -z "$detail" ]; then
    # Process the ha status
    proc_ha_status___new   
else
    proc_lssam
fi


