#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

####### NOTE NOTE NOTE NOTE ######
# Any common routines that you run from this procedure will come from
# the "old release" via /opt.  They do NOT come from /spare so don't
# get yourself confused!!!!
##################################

load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh

DEST_PRE=/spare
DEST_MIRROR0=$DEST_PRE/mirror0
DEST_OPT=$DEST_PRE/opt
DEST_VAR=$DEST_PRE/var
DEST_HOME=$DEST_PRE/home
DEST_ROOT=$DEST_PRE

DRBD_SOURCE=/etc/drbd.conf

RSCT_BKUP=/var/ct.backup

cpy_ha_config () {
	debug "cp -a $SMHA_CONFIGDIR ${DEST_ROOT}${SMHA_DIR}"
	cp -a $SMHA_CONFIGDIR ${DEST_ROOT}${SMHA_DIR}
	
	# Remove unwanted files
	rm -f ${DEST_ROOT}$MD_FILE
	rm -f ${DEST_ROOT}${SMHA_PAUSE_FILE}-*
	rm -rf ${DEST_ROOT}$SMHA_CONFIGDIR/.mutex*
	
}

cpy_ha_persist () {
    debug "cp -a $SMHA_DIR/persist $DEST_ROOT/$SMHA_DIR"
    nice -n 10 cp -a $SMHA_DIR/persist $DEST_ROOT/$SMHA_DIR
        
}

cpy_ha_logs () {
    debug "cp -a $LOG_DIR $DEST_ROOT/$SMHA_DIR"
    nice -n 10 cp -a $LOG_DIR ${DEST_ROOT}${SMHA_DIR}
        
}

#cpy_tiv_guid () {
	# Not sure we need this since VMI copies /etc/ files....
#    if [ -f /etc/ibm/tivoli/AM.guid ]; then
#        debug "cp -a /etc/ibm/tivoli/AM.guid $DEST_ROOT/etc/ibm/tivoli/AM.guid"
#        cp -a /etc/ibm/tivoli/AM.guid $DEST_ROOT/etc/ibm/tivoli/AM.guid
#    fi
#        
#}


cpy_drbd_info () {
    local i
    local j
    local sed_line=
    local TMPFILE=/tmp/smhaupgrd.$$

    debug "cp drbd info"

    on_host=$HOST

    $GETMIRROR_CMD > $TMPFILE

    i=0
    while read device mirror
    do
        devices[$i]=$device
        mirrors[$i]=$mirror
        i=$(( i + 1 ))
    done < $TMPFILE

    debug "devices => ${devices[*]}"
    debug "mirrors => ${mirrors[*]}"

    $GETMIRROR_CMD -s > $TMPFILE

    while read device mirror
    do
        debug "device=$device  mirror=$mirror"

        j=0
        for (( j=0; j < i; j++ ))
        do
            if [ "${mirrors[$j]}" = "$mirror" ]
            then
                if [ "${devices[$j]}" != "$device" ]
                then
                    debug "drbd.conf conversion; from ${devices[$j]} to $device"

                    sed_line_section="s#disk ${devices[$j]}#disk ${device}#g"

                    if [ -z "$sed_line" ]; then
                        sed_line="$sed_line_section"
                    else
                        sed_line="${sed_line};${sed_line_section}"
                    fi
                fi
                break
            fi
        done
    done < $TMPFILE
    rm -f $TMPFILE

    debug "drbd sed line is: $sed_line"
    sed "/on ${on_host}/ { N
    N
    N
    N
    N
    $sed_line}" /etc/drbd.conf >${DEST_ROOT}/etc/drbd.conf

    cp -a /etc/drbd.conf $DEST_ROOT/etc/drbd.conf.spare
}


bk_rsct () {
	
	if [ ! -d $RSCT_BKUP ]; then
        bkup_out=`nice -n 10 /usr/sbin/rsct/bin/ctbackup 2>&1` 
        # Remove the old copy
        rm -rf ${DEST_ROOT}${RSCT_BKUP}
        # Move the new copy to the spare
        nice -n 10 mv $RSCT_BKUP $DEST_VAR
        debug "RSCT BKUP: $bkup_out"
	fi
}

bk_ha_ssl_keys () {

    if [ -e /root/.ssh/authorized_keys2 ]; then
        cp -a /root/.ssh/authorized_keys2 /spare/root/.ssh/
    fi
}

###################################
## Main code                     ##
###################################
RC=0
debug " enter"

bk_rsct
bk_ha_ssl_keys
#cpy_tiv_guid

is_configured
if [ $? -eq 0 ]; then
    debug "HA is not configured."
else
   	# Originally the code was creating the .drbd.upgrade flag file
   	# in smhaupgrd_def.sh (which is run after reboot on the way back up).
   	# That would cause the flag file to get created on both active and passive
   	# nodes (remember that for GA1 you only upgraded active and you reinstalled
   	# the passive).  The flag file is used by the smhatsadrbd.sh script to determine
   	# which way the replication should happen.  We ONLY want it on the 
   	# active node, so moving it here.
   	is_passive
   	rc=$?
   	debug "is_passive returned $rc"
   	if [ $rc -eq 0 ]; then
   		# active node.  create the flag file in /opt.../ha/config dir and
   		# let it get copied to /spare with the rest.
   		debug "creating $DRBD_UPGRADE_FLAG"
   		touch $DRBD_UPGRADE_FLAG
   	fi
   	
    ## Run through the items to copy to the spare disk    
    cpy_ha_config
    cpy_ha_persist
    cpy_drbd_info

fi

# Copy the logs last so we have the most recent info
cpy_ha_logs

# work around a CSDA bug where /etc/services ends up having
# 600 permissions on the passive node, so DB2 won't start
/bin/chmod 644 /spare/etc/services

debug " exit; RC=$RC"
exit $RC
